#!/usr/bin/env python
#-*- coding:utf-8 -*-

###########################################################################
# Eole NG - 2011
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# creole_serv.py
# serveur XMLRPC pour accès aux données de configuration CREOLE
#
###########################################################################

from twisted.internet import reactor
from twisted.web import xmlrpc
from twisted.python import log
from os import seteuid, setegid, getuid, getgid
import json, os, traceback

from creole.cfgparser import EoleDict
from creole.config import eoledirs, configeol

#UID de l'utilisateur nobody
NOBODY_UID = 65534
#GID de l'utilisateur nobody
NOBODY_GID = 65534

class CreoleRPC(xmlrpc.XMLRPC):

    def __init__(self):
        self.uid=getuid()
        self.gid=getgid()
        self._drop_privileges()
        self.last_modified = None
        self.conf_file = configeol
        xmlrpc.XMLRPC.__init__(self)
        reactor.callWhenRunning(self.init_creole)

    def _drop_privileges(self):
        """
            drop de privileges vers nobody/nobody
        """
        setegid(NOBODY_GID)
        seteuid(NOBODY_UID)

    def _gain_privileges(self):
        """
            on récupère les uid et gid initiaux
        """
        seteuid(self.uid)
        setegid(self.gid)

    def init_creole(self):
        log.msg("CreoleRPC: lecture des dictionnaires")
        self.dico = self.load_dict()
        self.update_values()
        log.msg("CreoleRPC: serveur initialisé")

    def load_dict(self):
        self._gain_privileges()
        try:
            dico = EoleDict(logfile="/dev/null")
            dico.read_dir(eoledirs)
        except:
            traceback.print_exc()
        self._drop_privileges()
        return dico

    def update_values(self, force_reload=False):
        """met à jour les valeurs présentes dans config.eol si il a été
        modifié depuis la dernière lecture
        renvoie False si le ficher n'a pas changé
        """
        self._gain_privileges()
        try:
            if os.path.isfile(self.conf_file):
                conf_modtime = os.stat(self.conf_file).st_mtime
                if not force_reload:
                    if self.last_modified and (conf_modtime <= self.last_modified):
                        self._drop_privileges()
                        return json.dumps(False)
                self.last_modified = conf_modtime
                if self.dico.load_values():
                    log.msg("CreoleRPC: Valeurs rechargées depuis %s" % self.conf_file)
                else:
                    log.msg("CreoleRPC: Erreur lors du chargement de %s" % self.conf_file)
            else:
                log.msg("CreoleRPC: Attention, le fichier %s n'a pas été trouvé" % self.conf_file)
        except:
            traceback.print_exc()
        self._drop_privileges()

    def xmlrpc_reload(self):
        self.dico = self.load_dict()
        self.update_values(force_reload=True)

    def xmlrpc_get(self, varname, force_reload=False):
        """renvoie la valeur d'une variable particulière
        """
        if varname in self.dico.variables:
            self.update_values(force_reload)
            return json.dumps(self.dico.get_value(varname))
        return json.dumps('')

    def xmlrpc_parse_dico(self, hide_vars=False, force_reload=False):
        """renvoie la valeur de toutes les variables sous
        forme d'un dictionnaire.
        hide_vars : n'affiche pas les variables 'hidden'
        """
        self.update_values(force_reload)
        return json.dumps(self.dico.todict(hide_vars))

    def xmlrpc_list_variables(self):
        """renvoie la liste des variables connues
        """
        return json.dumps(self.dico.variables.keys())

    def xmlrpc_ping(self):
        """fonction pour tester la disponibilité du serveur
        """
        return json.dumps(True)
